@@ -362,6 +362,7 @@ void signal_handler(int sig)
 			need_died_procs_check = 1;
 			break;
 
+		case SIGINT:
 		case SIGHUP:
 		case SIGTERM:
 			do_exit();
@@ -398,8 +399,9 @@ void select_display_start_and_end(char *string, char mode, int wrap_offset, int
 	}
 	else if (mode == 'l')
 	{
-		*prt_end = min(nbytes, win_width);
-		*disp_end = win_width;
+		*prt_start= min(wrap_offset, nbytes);
+		*prt_end = min(nbytes, win_width + wrap_offset);
+		*disp_end = min(win_width, nbytes - *prt_start);
 	}
 	else if (mode == 'r')
 	{
@@ -697,6 +699,9 @@ void do_color_print(proginfo *cur, char *use_string, int prt_start, int prt_end,
 			break;
 	}
 
+	if (prt_start == prt_end)       /* scrolled out line */
+		wprintw(win -> win, "\n");
+
 	if (cdev.attrs != -1)
 		myattr_off(win, cdev);
 
@@ -772,6 +777,7 @@ void color_print(int f_index, NEWWIN *win, proginfo *cur, char *string, regmatch
 	}
 
 	/* add a label */
+	LOG("Label\n");
 	if (cur -> label != NULL && (cur -> label)[0])
 	{
 		mx -= wprintw(win -> win, "%s", cur -> label);
@@ -805,6 +811,7 @@ void color_print(int f_index, NEWWIN *win, proginfo *cur, char *string, regmatch
 	}
 
 	/* select start and end of string to display */
+	LOG("lwo: %d, mx: %d, ps: %d, pe: %d, de: %d\n", cur -> line_wrap_offset, mx, prt_start, prt_end, disp_end);
 	select_display_start_and_end(USE_IF_SET(use_string, string), force_to_winwidth == MY_TRUE?'l':cur -> line_wrap, cur -> line_wrap_offset, mx, &prt_start, &prt_end, &disp_end);
 
 	if (prt_start == 0 && start_at_offset > 0)
@@ -814,6 +821,8 @@ void color_print(int f_index, NEWWIN *win, proginfo *cur, char *string, regmatch
 	}
 
 	/* and display on terminal */
+	LOG("ps: %d, pe: %d, de: %d\n", prt_start, prt_end, disp_end);
+	LOG("%s\n", USE_IF_SET(use_string, string));
 	do_color_print(cur, USE_IF_SET(use_string, string), prt_start, prt_end, disp_end, cmatches, n_cmatches, has_merge_colors, reverse, matches, matching_regex, use_regex, win);
 
 	myattr_off(win, cdev);
@@ -1152,6 +1161,38 @@ void redirect(proginfo *cur, char *data, int n_bytes, mybool_t is_filtered)
 	}
 }
 
+void delete_all_markerlines(void)
+{
+	int index;
+
+	for(index=0; index<nfd; index++)
+	{
+		int loop, ndeleted = 0;
+		buffer cur_lb;
+
+		memset(&cur_lb, 0x00, sizeof(buffer));
+
+		for(loop=0; loop<lb[index].curpos; loop++)
+		{
+			if ((lb[index].be)[loop].Bline == NULL)
+			{
+				ndeleted++;
+				continue;
+			}
+
+			cur_lb.be = myrealloc(cur_lb.be, (cur_lb.curpos + 1) * sizeof(buffered_entry), __FILE__, __PRETTY_FUNCTION__, __LINE__);
+			cur_lb.be[cur_lb.curpos].Bline = (lb[index].be)[loop].Bline;
+			cur_lb.be[cur_lb.curpos].pi    = (lb[index].be)[loop].pi;
+			cur_lb.be[cur_lb.curpos].ts    = (lb[index].be)[loop].ts;
+			cur_lb.curpos++;
+		}
+
+		delete_be_in_buffer(&lb[index]);
+		lb[index].be = cur_lb.be;
+		lb[index].curpos -= ndeleted;
+	}
+}
+
 int add_marker_if_changed(int f_index, proginfo *cur)
 {
 	int changed = 0;
@@ -2319,7 +2360,7 @@ void init_check_for_mail(void)
 			printf("Could not determine size of file '%s' (which is supposed to be ", mail_spool_file);
 			printf("your mailfile): mail-check is disabled.\n");
 			printf("You can prevent this message by adding the line 'check_mail:0' ");
-			printf("in /etc/multitail.conf or in .multitailrc in your home-directory.\n\n");
+			printf("in "CONFIG_FILE" or in .multitailrc in your home-directory.\n\n");
 			printf("Press enter to continue...");
 			fflush(NULL);
 			getchar();
@@ -2362,6 +2403,12 @@ void main_loop(void)
 			{
 				break;
 			}
+			else if (c == 'Z')	/* clear all marker lines */
+			{
+				delete_all_markerlines();
+				do_refresh = 2;
+				continue;
+			}
 			else if (uc == 'A')
 			{
 				if (add_window())
@@ -2508,6 +2555,13 @@ void main_loop(void)
 				else
 					wrong_key();
 			}
+			else if (uc == 13)
+			{
+				int loop;
+
+				for(loop=0; loop<nfd; loop++)
+					add_markerline(loop, NULL, MARKER_REGULAR, NULL);
+			}
 			else if (uc == 'K')
 			{
 				terminal_mode();
@@ -2567,6 +2621,11 @@ void main_loop(void)
 			{
 				clear_all_buffers();
 			}
+			else if (c == KEY_LEFT || c == KEY_RIGHT)
+			{
+				horizontal_scroll(c);
+				do_refresh = 2;
+			}
 			else
 			{
 				if (exec_bind(c))
@@ -2617,6 +2676,7 @@ int main(int argc, char *argv[])
 	time(&mt_started);
 
 	set_signal(SIGTERM, signal_handler, "SIGTERM");
+	set_signal(SIGINT,  signal_handler, "SIGINT" );
 	set_signal(SIGHUP,  signal_handler, "SIGHUP" );
 	set_signal(SIGCHLD, signal_handler, "SIGCHLD");
 
@@ -3013,9 +3073,9 @@ int process_global_keys(int what_help, NEWWIN *popup, char cursor_shift)
 	{
 		show_help(what_help);
 	}
-	else if ((cursor_shift == 0 && c == KEY_LEFT) || (cursor_shift == 1 && c == KEY_SLEFT))
+	else if (popup != NULL && ((cursor_shift == 0 && c == KEY_LEFT) || (cursor_shift == 1 && c == KEY_SLEFT)))
 	{
-		if (popup != NULL && popup -> x_off > 0)
+		if (popup -> x_off > 0)
 		{
 			popup -> x_off--;
 			move_panel(popup -> pwin, popup -> y_off, popup -> x_off);
@@ -3035,9 +3095,9 @@ int process_global_keys(int what_help, NEWWIN *popup, char cursor_shift)
 		else
 			wrong_key();
 	}
-	else if ((cursor_shift == 0 && c == KEY_RIGHT) || (cursor_shift == 1 && c == KEY_SRIGHT))
+	else if (popup != NULL && ((cursor_shift == 0 && c == KEY_RIGHT) || (cursor_shift == 1 && c == KEY_SRIGHT)))
 	{
-		if (popup != NULL && (popup -> x_off + popup -> width) < max_x)
+		if ((popup -> x_off + popup -> width) < max_x)
 		{
 			popup -> x_off++;
 			move_panel(popup -> pwin, popup -> y_off, popup -> x_off);
@@ -3349,20 +3409,6 @@ int wait_for_keypress(int what_help, double max_wait, NEWWIN *popup, char cursor
 		if (max_wait != 0.0 && (get_ts() - max_wait_start) >= max_wait)
 			break;
 
-		/* see if any processes have died (processes started
-		 * by matchin regular expressions)
-		 */
-		if (need_died_procs_check)
-		{
-			need_died_procs_check = 0;
-
-			if (check_for_died_processes())
-			{
-				c = -1;
-				break;
-			}
-		}
-
 		if (got_sigusr1)
 		{
 			got_sigusr1 = 0;
@@ -3492,6 +3538,20 @@ int wait_for_keypress(int what_help, double max_wait, NEWWIN *popup, char cursor
 			}
 		}
 
+		/* see if any processes have died (processes started
+		 * by matchin regular expressions)
+		 */
+		if (need_died_procs_check)
+		{
+			need_died_procs_check = 0;
+
+			if (check_for_died_processes())
+			{
+				c = -1;
+				break;
+			}
+		}
+
 closed_window:
 
 		/* any window changed? then we may want to update the terminal window header */
