@@ -221,6 +221,30 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 	int line_offset = 0;
 	char show_winnr = default_sb_showwinnr;
 	mybool_t case_insensitive = re_case_insensitive;
+	buffer cur_lb;
+	int loop;
+
+	memset(&cur_lb, 0x00, sizeof(cur_lb));
+
+	for(loop=0; loop<pbuf -> curpos; loop++)
+	{
+		if ((pbuf -> be)[loop].Bline == NULL)
+			continue;
+
+		cur_lb.be = myrealloc(cur_lb.be, (cur_lb.curpos + 1) * sizeof(buffered_entry), __FILE__, __PRETTY_FUNCTION__, __LINE__);
+		cur_lb.be[cur_lb.curpos].pi    = (pbuf -> be)[loop].pi;
+		if ((pbuf -> be)[loop].pi != NULL && (!IS_MARKERLINE((pbuf -> be)[loop].pi)) && (pbuf -> be)[loop].pi -> cdef.term_emul != TERM_IGNORE)
+		{
+			color_offset_in_line *cmatches;
+			int n_cmatches;
+			cur_lb.be[cur_lb.curpos].Bline = emulate_terminal((pbuf -> be)[loop].Bline, &cmatches, &n_cmatches);
+			myfree(cmatches);
+		}
+		else
+			cur_lb.be[cur_lb.curpos].Bline = strdup((pbuf -> be)[loop].Bline);
+		cur_lb.be[cur_lb.curpos].ts    = (pbuf -> be)[loop].ts;
+		cur_lb.curpos++;
+	}
 
 	LOG("---\n");
 	if (global_highlight_str)
@@ -243,7 +267,7 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 			int lines_used = 0;
 
 			ui_inverse_on(mywin1);
-			mvwprintw(mywin1 -> win, nlines + 1, 1, "%s - %d buffered lines", shorten_filename(header, max(24, ncols - 24)), pbuf -> curpos);
+			mvwprintw(mywin1 -> win, nlines + 1, 1, "%s - %d buffered lines", shorten_filename(header, max(24, ncols - 24)), cur_lb.curpos);
 			ui_inverse_off(mywin1);
 
 			if (!no_linewrap) ui_inverse_on(mywin1);
@@ -255,11 +279,11 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 			if (!no_linewrap && line_offset > 0)
 			{
 				int temp_line_offset = line_offset;
-				int n_chars_to_display_left = strlen((pbuf -> be)[offset].Bline) - temp_line_offset;
+				int n_chars_to_display_left = strlen((cur_lb.be)[offset].Bline) - temp_line_offset;
 
 				while(lines_used < nlines && n_chars_to_display_left > 0)
 				{
-					scrollback_displayline(winnrs?winnrs[offset]:window_nr, mywin2, pbuf, offset, lines_used, temp_line_offset, 1, show_winnr);
+					scrollback_displayline(winnrs?winnrs[offset]:window_nr, mywin2, &cur_lb, offset, lines_used, temp_line_offset, 1, show_winnr);
 
 					temp_line_offset += ncols;
 					n_chars_to_display_left -= ncols;
@@ -270,13 +294,13 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 				index++;
 			}
 
-			for(;(offset + index) < pbuf -> curpos && lines_used < nlines;)
+			for(;(offset + index) < cur_lb.curpos && lines_used < nlines;)
 			{
-				int lines_needed = get_lines_needed((pbuf -> be)[offset + index].Bline, ncols);
+				int lines_needed = get_lines_needed((cur_lb.be)[offset + index].Bline, ncols);
 
 				if (no_linewrap || lines_needed == 1)
 				{
-					scrollback_displayline(winnrs?winnrs[offset + index]:window_nr, mywin2, pbuf, offset + index, lines_used, no_linewrap?line_offset:0, no_linewrap, show_winnr);
+					scrollback_displayline(winnrs?winnrs[offset + index]:window_nr, mywin2, &cur_lb, offset + index, lines_used, no_linewrap?line_offset:0, no_linewrap, show_winnr);
 					lines_used++;
 				}
 				else
@@ -285,7 +309,7 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 
 					while(lines_used < nlines && lines_needed > 0)
 					{
-						scrollback_displayline(winnrs?winnrs[offset + index]:window_nr, mywin2, pbuf, offset + index, lines_used, cur_line_offset, 1, show_winnr);
+						scrollback_displayline(winnrs?winnrs[offset + index]:window_nr, mywin2, &cur_lb, offset + index, lines_used, cur_line_offset, 1, show_winnr);
 						cur_line_offset += ncols;
 						lines_used++;
 						lines_needed--;
@@ -398,13 +422,13 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 			{
 				offset--;
 
-				line_offset = (get_lines_needed((pbuf -> be)[offset].Bline, ncols) - 1) * ncols;
+				line_offset = (get_lines_needed((cur_lb.be)[offset].Bline, ncols) - 1) * ncols;
 			}
 
 			wmove(mywin2 -> win, 0, 0);
 			winsdelln(mywin2 -> win, 1);
 
-			scrollback_displayline(winnrs?winnrs[offset]:window_nr, mywin2, pbuf, offset, 0, line_offset, no_linewrap, show_winnr);
+			scrollback_displayline(winnrs?winnrs[offset]:window_nr, mywin2, &cur_lb, offset, 0, line_offset, no_linewrap, show_winnr);
 
 			redraw = 1;
 		}
@@ -415,19 +439,19 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 					c == 14  || /* ^n */
 					c == 13  ||
 					c == KEY_ENTER)
-				&& offset < (pbuf -> curpos - 1))
+				&& offset < (cur_lb.curpos - 1))
 		{
 			if (no_linewrap)
 			{
 				offset++;
 			}
-			else if (strlen((pbuf -> be)[offset].Bline) > (line_offset + ncols))
+			else if (strlen((cur_lb.be)[offset].Bline) > (line_offset + ncols))
 			{
 				line_offset += ncols;
 			}
-			else if (offset < (pbuf -> curpos - 1))
+			else if (offset < (cur_lb.curpos - 1))
 			{
-				if (strlen((pbuf -> be)[offset].Bline) > (line_offset + ncols))
+				if (strlen((cur_lb.be)[offset].Bline) > (line_offset + ncols))
 					line_offset += ncols;
 				else
 				{
@@ -446,23 +470,23 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 					c == 'z' ||
 					c == 'u' ||
 					c == ('U' - 65 + 1))   /* ^u */
-				&& offset < (pbuf -> curpos - 1))
+				&& offset < (cur_lb.curpos - 1))
 		{
 			if (no_linewrap)
 			{
 				offset += nlines;
-				if (offset >= pbuf -> curpos)
-					offset = pbuf -> curpos - 1;
+				if (offset >= cur_lb.curpos)
+					offset = cur_lb.curpos - 1;
 			}
 			else
 			{
 				int n_lines_to_move = nlines;
 
-				while(n_lines_to_move > 0 && offset < (pbuf -> curpos))
+				while(n_lines_to_move > 0 && offset < (cur_lb.curpos))
 				{
 					if (line_offset > 0)
 					{
-						if (line_offset + ncols >= strlen((pbuf -> be)[offset].Bline))
+						if (line_offset + ncols >= strlen((cur_lb.be)[offset].Bline))
 						{
 							line_offset = 0;
 							offset++;
@@ -476,7 +500,7 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 					}
 					else
 					{
-						n_lines_to_move -= get_lines_needed((pbuf -> be)[offset].Bline, ncols);
+						n_lines_to_move -= get_lines_needed((cur_lb.be)[offset].Bline, ncols);
 						offset++;
 					}
 				}
@@ -512,11 +536,11 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 				{
 					offset--;
 
-					n_lines_to_move -= get_lines_needed((pbuf -> be)[offset].Bline, ncols);
+					n_lines_to_move -= get_lines_needed((cur_lb.be)[offset].Bline, ncols);
 
 					if (n_lines_to_move < 0)
 					{
-						line_offset = (get_lines_needed((pbuf -> be)[offset].Bline, ncols) + n_lines_to_move) * ncols;
+						line_offset = (get_lines_needed((cur_lb.be)[offset].Bline, ncols) + n_lines_to_move) * ncols;
 					}
 				}
 			}
@@ -536,9 +560,9 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 					c == 'G' ||
 					c == '>' ||
 					c == KEY_SEND)
-				&& offset < (pbuf -> curpos - 1))
+				&& offset < (cur_lb. curpos - 1))
 		{
-			offset = pbuf -> curpos - 1;
+			offset = cur_lb. curpos - 1;
 			redraw = 2;
 		}
 		else if (uc == 'R' || c == ('R' - 65 + 1) || c == ('L' - 65 + 1) || c == KEY_REFRESH)
@@ -581,7 +605,7 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 			{
 				if (scrollback_search_new_window)
 				{
-					if (scrollback_search_to_new_window(pbuf, header, find, case_insensitive) == -1)
+					if (scrollback_search_to_new_window(&cur_lb, header, find, case_insensitive) == -1)
 					{
 						/* cascaded close */
 						rc = -1;
@@ -598,7 +622,7 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 					myfree(global_highlight_str);
 					global_highlight_str = NULL;
 
-					new_f_index = find_string(pbuf, find, 0, direction, case_insensitive);
+					new_f_index = find_string(&cur_lb, find, 0, direction, case_insensitive);
 					if (new_f_index == -1)
 					{
 						wrong_key();
@@ -617,7 +641,7 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 			{
 				char direction = (c == 'n' || c == KEY_NEXT) ? 1 : -1;
 				int start_offset = offset + direction;
-				int new_f_index = find_string(pbuf, find, start_offset, direction, case_insensitive);
+				int new_f_index = find_string(&cur_lb, find, start_offset, direction, case_insensitive);
 				if (new_f_index == -1)
 				{
 					wrong_key();
@@ -636,7 +660,7 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 		}
 		else if (c == 's' || c == KEY_SAVE)
 		{
-			scrollback_savefile(pbuf);
+			scrollback_savefile(&cur_lb);
 			redraw = 2;	/* force redraw */
 		}
 		else if (c == 'h')
@@ -672,6 +696,8 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 
 	myfree(find);
 
+	delete_be_in_buffer(&cur_lb);
+
 	return rc;
 }
 
