@@ -4,6 +4,8 @@
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
 
 #include "mt.h"
 #include "help.h"
@@ -99,7 +101,7 @@ int edit_stripper_edit_field(NEWWIN *win, char **del, int *col_nr)
 
 	if (new_del != NULL && new_col_nr != NULL)
 	{
-		*del = mystrdup(new_del, "edit_stripper_edit_field: del");
+		*del = mystrdup(new_del, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 		*col_nr = atoi(new_col_nr);
 		changed = 1;
 	}
@@ -143,7 +145,7 @@ int edit_strippers(void)
 		char linebuf[68 + 1];
 
 		werase(mywin -> win);
-		wborder(mywin -> win, box_left_side, box_right_side, box_top_side, box_bottom_side, box_top_left_hand_corner, box_top_right_hand_corner, box_bottom_left_hand_corner, box_bottom_right_hand_corner);
+		draw_border(mywin);
 		win_header(mywin, "Edit strip regular expressions");
 		mvwprintw(mywin -> win, 2, 2, "%s", cur -> filename);
 		escape_print(mywin, 3, 2, "^a^dd, ^e^dit, ^d^elete, ^q^uit");
@@ -161,6 +163,7 @@ int edit_strippers(void)
 
 			switch((cur -> pstrip)[loop].type)
 			{
+				case STRIP_KEEP_SUBSTR:
 				case STRIP_TYPE_REGEXP:
 					strncpy(linebuf, (cur -> pstrip)[loop].regex_str, 55);
 					linebuf[55] =  0x00;
@@ -243,21 +246,21 @@ int edit_strippers(void)
 			}
 
 			/* ask type */
-			escape_print(mywin, 15, 2, "Type? reg.^e^xp., ^r^ange, ^c^olumn");
+			escape_print(mywin, 15, 2, "reg.^e^xp., ^r^ange, ^c^olumn, ^S^ubstrings");
 			mydoupdate();
 			for(;;)
 			{
 				c = toupper(wait_for_keypress(HELP_STRIPPER_TYPE, 0, mywin, 0));
 
-				if (c == 'E' || c == 'R' || c == 'C' ||
-						c == abort_key   || c == 'Q' || c == 'X')
+				if (c == 'E' || c == 'R' || c == 'C' || c == 'S' ||
+						c == abort_key || c == 'Q' || c == 'X')
 					break;
 
 				wrong_key();
 			}
 			if (c == abort_key || c == 'Q' || c == 'X') continue;
 
-			if (c == 'E')
+			if (c == 'E' || c == 'S')
 			{
 				/* user did not abort edit? */
 				if (edit_stripper_edit_regexp(mywin, &regex_str, &case_insensitive))
@@ -278,11 +281,11 @@ int edit_strippers(void)
 			changed = 1;
 
 			cur_strip = cur -> n_strip++;
-			cur -> pstrip = (strip_t *)myrealloc(cur -> pstrip, cur -> n_strip * sizeof(strip_t), "edit_strippers: list of strippers");
+			cur -> pstrip = (strip_t *)myrealloc(cur -> pstrip, cur -> n_strip * sizeof(strip_t), __FILE__, __PRETTY_FUNCTION__, __LINE__);
 			memset(&(cur -> pstrip)[cur_strip], 0x00, sizeof(strip_t));
 
 			/* compile */
-			if (c == 'E')
+			if (c == 'E' || c == 'S')
 			{
 				if ((rc = regcomp(&(cur -> pstrip)[cur_strip].regex, regex_str, REG_EXTENDED)))
 				{
@@ -297,7 +300,7 @@ int edit_strippers(void)
 				{
 					/* compilation went well, remember everything */
 					(cur -> pstrip)[cur_strip].regex_str = regex_str;
-					(cur -> pstrip)[cur_strip].type = STRIP_TYPE_REGEXP;
+					(cur -> pstrip)[cur_strip].type = (c == 'E' ? STRIP_TYPE_REGEXP : STRIP_KEEP_SUBSTR);
 				}
 			}
 			else if (c == 'R')
@@ -354,7 +357,8 @@ int edit_strippers(void)
 		{
 			if (cur -> n_strip > 0)
 			{
-				if ((cur -> pstrip)[cur_strip].type == STRIP_TYPE_REGEXP)
+				if ((cur -> pstrip)[cur_strip].type == STRIP_TYPE_REGEXP ||
+				    (cur -> pstrip)[cur_strip].type == STRIP_KEEP_SUBSTR)
 				{
 					if (edit_stripper_edit_regexp(mywin, &(cur -> pstrip)[cur_strip].regex_str, &case_insensitive))
 						changed = 1;
@@ -380,6 +384,11 @@ int edit_strippers(void)
 	return changed;
 }
 
+void zero_str(char *what, int start, int end)
+{
+	memset(&what[start], 0x00, end-start);
+}
+
 char do_strip_re(regex_t *pre, int *match_count, char *in, char *strip_what)
 {
 	char changed = 0;
@@ -400,7 +409,7 @@ char do_strip_re(regex_t *pre, int *match_count, char *in, char *strip_what)
 						matches[match_i].rm_eo == -1)
 					break;
 
-				memset(&strip_what[matches[match_i].rm_so + search_offset], 0x00, matches[match_i].rm_eo - matches[match_i].rm_so);
+				zero_str(strip_what, matches[match_i].rm_so + search_offset, matches[match_i].rm_eo + search_offset);
 				changed = 1;
 
 				new_offset = matches[match_i].rm_eo + search_offset;
@@ -416,6 +425,36 @@ char do_strip_re(regex_t *pre, int *match_count, char *in, char *strip_what)
 	return changed;
 }
 
+char do_keep_re(regex_t *pre, int *match_count, char *in, char *strip_what)
+{
+	char changed = 0;
+	regmatch_t matches[MAX_N_RE_MATCHES];
+	int last_offset = 0;
+
+	if (regexec(pre, in, MAX_N_RE_MATCHES, matches, 0) != REG_NOMATCH)
+	{
+		int match_i;
+
+		for(match_i=1; match_i<MAX_N_RE_MATCHES; match_i++)
+		{
+			if (matches[match_i].rm_so == -1 || matches[match_i].rm_eo == -1)
+				break;
+
+			zero_str(strip_what, last_offset, matches[match_i].rm_so);
+			last_offset = matches[match_i].rm_eo;
+		}
+
+		if (last_offset != 0)
+		{
+			zero_str(strip_what, last_offset, strlen(in));
+		}
+
+		changed = 1;
+	}
+
+	return changed;
+}
+
 char do_strip_column(char *delimiter, int col_nr, char *in, char *strip_what)
 {
 	char changed = 0;
@@ -468,8 +507,8 @@ char * do_strip(proginfo *cur, char *in)
 	if (len == 0)
 		return NULL;
 
-	strip_what = (char *)mymalloc(len, "do_strip: strip_what");
-	new_string = (char *)mymalloc(len + 1, "do_strip: new string");
+	strip_what = (char *)mymalloc(len,     __FILE__, __PRETTY_FUNCTION__, __LINE__);
+	new_string = (char *)mymalloc(len + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
 	memset(strip_what, 0x01, len);
 
@@ -488,8 +527,10 @@ char * do_strip(proginfo *cur, char *in)
 		{
 			changed |= do_strip_column((cur -> pstrip)[loop].del, (cur -> pstrip)[loop].col_nr, in, strip_what);
 		}
-		else
-			error_exit("do_strip: unknown strip-command detected (%d)\n", (cur -> pstrip)[loop].type);
+		else if ((cur -> pstrip)[loop].type == STRIP_KEEP_SUBSTR)
+		{
+			changed |= do_keep_re(&(cur -> pstrip)[loop].regex, &(cur -> pstrip)[loop].match_count, in, strip_what);
+		}
 	}
 
 	if (changed)
