@@ -953,33 +953,12 @@ int get_iat_element(int_array_t *piat, int index)
 
 char *gethome(char *user)
 {
-	char *path = NULL;
-
-	for(;;)
-	{
-		struct passwd *p = getpwent();
-
-		if (!p)
-			break;
-
-		if (user != NULL)
-		{
-			if (strcmp(p -> pw_name, user) == 0)
-			{
-				path = mystrdup(p -> pw_dir, __FILE__, __PRETTY_FUNCTION__, __LINE__);
-				break;
-			}
-		}
-		else if (p -> pw_uid == getuid())
-		{
-			path = mystrdup(p -> pw_dir, __FILE__, __PRETTY_FUNCTION__, __LINE__);
-			break;
-		}
-	}
-
-	endpwent();
-
-	return path;
+	struct passwd *pw;
+	if (user)
+		pw = getpwnam(user);
+	else
+		pw = getpwuid(getuid());
+	return mystrdup(pw->pw_dir, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 }
 
 char *myrealpath(char *in)
